<?php

namespace Drupal\twitter_api_block\Plugin\Block;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\UncacheableDependencyTrait;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a 'TwitterBlock' block.
 *
 * @see https://github.com/J7mbo/twitter-api-php
 */
class TwitterBlockBase extends BlockBase {

  use UncacheableDependencyTrait;

  /**
   *
   */
  protected $settings;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    // Twitter credentials.
    $form['api'] = [
      '#type' => 'details',
      '#title' => $this->t('Twitter credentials'),
      '#description' => $this->t('Create a new application at <a href="https://apps.twitter.com/" target="_blank">https://apps.twitter.com/</a>.'),
      '#collapsible' => TRUE,
      '#open' => TRUE,
    ];
    $form['api']['consumer_key'] = [
      '#title' => t('Consumer Key'),
      '#type' => 'textfield',
      '#default_value' => isset($config['api']['consumer_key']) ? $config['api']['consumer_key'] : NULL,
      '#required' => TRUE,
    ];
    $form['api']['consumer_secret'] = [
      '#title' => t('Consumer Secret'),
      '#type' => 'textfield',
      '#default_value' => isset($config['api']['consumer_secret']) ? $config['api']['consumer_secret'] : NULL,
      '#required' => TRUE,
    ];
    $form['api']['oauth_access_token'] = [
      '#title' => t('OAuth Access Token'),
      '#type' => 'textfield',
      '#default_value' => isset($config['api']['oauth_access_token']) ? $config['api']['oauth_access_token'] : NULL,
      '#required' => TRUE,
    ];
    $form['api']['oauth_access_token_secret'] = [
      '#title' => t('OAuth Access Token Secret'),
      '#type' => 'textfield',
      '#default_value' => isset($config['api']['oauth_access_token_secret']) ? $config['api']['oauth_access_token_secret'] : NULL,
      '#required' => TRUE,
    ];
    // Basic block options.
    $form['options'] = [
      '#type' => 'details',
      '#title' => $this->t('Options'),
      '#collapsible' => false,
      '#open' => TRUE,
    ];
    $form['options']['count'] = [
      '#type' => 'number',
      '#title' => $this->t("Number of tweets"),
      '#default_value' => isset($config['options']['count']) ? $config['options']['count'] : 3,
      '#required' => TRUE,
    ];
    // Cache-related.
    $form['options']['disable_cache'] = [
      '#type' => 'checkbox',
      '#title' => $this->t("Disable cache"),
      '#description' => $this->t("" .
        "If checked, this block will trigger the `page_cache_kill_switch` when displayed." .
        "This has impact on your site's performance." .
        "Use with caution."),
      '#default_value' => isset($config['options']['replies']) ? $config['options']['replies'] : 0,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    // Save our custom settings when the form is submitted.
    foreach ($form_state->getValues() as $key => $value) {
      $this->setConfigurationValue($key, $value);
    };
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    $config = $this->getConfiguration();

    // Stop now if credentials are not setup.
    if (
      NULL == $config['api']['consumer_key'] ||
      NULL == $config['api']['consumer_secret'] ||
      NULL == $config['api']['oauth_access_token'] ||
      NULL == $config['api']['oauth_access_token_secret']
    ) {
      return [];
    }

    // Get credentials.
    $this->settings = [
      'consumer_key' => $config['api']['consumer_key'],
      'consumer_secret' => $config['api']['consumer_secret'],
      'oauth_access_token' => $config['api']['oauth_access_token'],
      'oauth_access_token_secret' => $config['api']['oauth_access_token_secret'],
    ];

    // Attach JS and styles.
    $build['#attached']['library'][] = 'twitter_api_block/base';

    // Cache depending on the current user.
    $build['#cache'] = [
      'contexts' => [
        'user',
      ],
    ];

    if (isset($this->configuration['disable_cache']) && $this->configuration['disable_cache']) {
      // Do not cache any page with this block on it.
      \Drupal::service('page_cache_kill_switch')->trigger();

      // Disable caching.
      $build['#cache'] = [
        'contexts' => [
          'user',
        ],
        'max-age' => 0,
      ];
    }

    return $build;
  }

  /**
   * Custom static function used in blocks to generate query parameters.
   */
  private static function getParameters($config) {
    return UrlHelper::buildQuery([]);
  }
}
